#!/bin/bash
# Script para monitoramento da utilizacao da memoria com /proc/meminfo
# Autores: Carlos Melo, Paulo Pereira e Ronierison Maciel
# {casm3,prps,rsm4}@cin.ufpe.br

tempo=0;
echo "MemFree;Buffers;Cached;SwapFree;" > log-mem-proc.csv
while [ $tempo -lt 300 ]
do
	mfree=$(cat /proc/meminfo | awk '/MemFree/{print $2}')
	buffer=$(cat /proc/meminfo | awk '/Buffers/{print $2}')
	# ^ antes da palavra Cached significa que a linha deve iniciar com Cached
	cach=$(cat /proc/meminfo | awk '/^Cached/{print $2}')
	swapfree=$(cat /proc/meminfo | awk '/SwapFree/{print $2}')
	echo "$mfree;$buffer;$cach;$swapfree;" >> log-mem-proc.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script monitoramento processo ex: chrome
tempo=0;
pid=$(ps aux | grep chrome | head -n 1 | awk '{print $2}')
echo "State;VmSize;VmHWM;VmRSS;" > log-process_proc.csv
while [ $tempo -lt 300 ]
do
	mfree=$(cat /proc/$pid/status | awk '/State/{print $2}')
	buffer=$(cat /proc/$pid/status | awk '/VmSize/{print $2}')
	cach=$(cat /proc/$pid/status | awk '/VmHWM/{print $2}')
	swapfree=$(cat /proc/$pid/status | awk '/VmRSS/{print $2}')
	echo "$mfree;$buffer;$cach;$swapfree;" >> log-process_proc.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script usando top
tempo=0;
echo "VIRT;RES;%CPU;%MEM;" > log-uptime-top.csv
while [ $tempo -lt 300 ]
do
	virt=$(top -b -n 1 | awk '/chrome/{print $5}' | head -n 1)
	res=$(top -b -n 1 | awk '/chrome/{print $6}' | head -n 1)
	cpu=$(top -b -n 1 | awk '/chrome/{print $9}' | head -n 1)
	mem=$(top -b -n 1 | awk '/chrome/{print $10}' | head -n 1)
	echo "$virt;$res;$cpu;$mem;" >> log-uptime-top.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script usando top e uptime
# monitorando o processo chrome, mem virtual, residente, consumo de cpu, qnts dias o computador ta ativo
tempo=0;
echo "TIME;DAYS;VIRT;RES;%CPU;%MEM;" > log-top.csv
while [ $tempo -lt 300 ]
do
    time=$(uptime | awk '{print $1}')
    days=$(uptime | awk '{print $3}')
	virt=$(top -b -n 1 | awk '/chrome/{print $5}' | head -n 1)
	res=$(top -b -n 1 | awk '/chrome/{print $6}' | head -n 1)
	cpu=$(top -b -n 1 | awk '/chrome/{print $9}' | head -n 1)
	mem=$(top -b -n 1 | awk '/chrome/{print $10}' | head -n 1)
	echo "$TIME;$days;$virt;$res;$cpu;$mem;" >> log-top.csv
	sleep 5
	tempo=$((tempo+5))
done

# Script vmstat
# monitorando processos ativos, memoria principal livre, blocos trazidos do disco para a memoria principal, bo, e setores lidos com sucesso do disco
tempo=0;
echo "PACTIVE;FREE;BI;BO;SECTOR;" > log-vmstat.csv
while [ $tempo -lt 300 ]
do
    pactive=$(vmstat | tail -n1 | awk '{print $1}')
    free=$(vmstat | tail -n1 | awk '{print $4}')
    bi=$(vmstat | tail -n1 | awk '{print $9}')
    bo=$(vmstat | tail -n1 | awk '{print $10}')
    sector=$(vmstat -d | awk '/sda/{print $4}')
	
	echo "$pactive;$free;$bi;$bo;$sector;" >> log-vmstat.csv
	sleep 5
	tempo=$((tempo+5))
done

# Script free
# monitorando a memoria principal
tempo=0;
echo "FREE;SHARED;TOTAL;" > log-free.csv
while [ $tempo -lt 300 ]
do
    free=$(free -m | awk '/Mem/{print $4}')
    shared=$(free -m | awk '/Mem/{print $5}')
    total=$(free -m | awk '/Mem/{print $2}')
	
	echo "$free;$shared;$total;" >> log-free.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script iostat
# monitorando o tempo e o tamanho que as requisicoes ficam na fila
tempo=0;
echo "R-AWAIT;RAREQ-SZ;W-AWAIT;WAREQ-SZ;" > log-iostat-sda.csv
while [ $tempo -lt 300 ]
do
    rawait=$(iostat -x | awk '/sda/{print $6}')
    rareqsz=$(iostat -x | awk '/sda/{print $7}')

    wawait=$(iostat -x | awk '/sda/{print $12}')
    wareqsz=$(iostat -x | awk '/sda/{print $13}')
    
	echo "$rawait;$rareqsz;$wawait;$wareqsz;" >> log-iostat-sda.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script mpstat
# monitorando o idle de cada nucleo do processador
tempo=0;
echo "IDLE-0;IDLE-1;IDLE-2;IDLE-3;" > log-mpstat.csv
while [ $tempo -lt 300 ]
do
    i0=$(mpstat -P 0 | tail -n 1 | awk '/sda/{print $13}')
    i1=$(mpstat -P 1 | tail -n 1 | awk '/sda/{print $13}')
    i2=$(mpstat -P 2 | tail -n 1 | awk '/sda/{print $13}')
    i3=$(mpstat -P 3 | tail -n 1 | awk '/sda/{print $13}')

    
	echo "$i0;$i1;$i2;$i3;" >> log-mpstat.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script pidstat
# monitorando dropbox: qual nucleo esta usando, minor fault, major fault, virtual size, residente size, kb_r, kb_w
tempo=0;
echo "CPU;MINFLT;MAJFLT;VSZ;RSS;KB-R;KB-W;" > log-pidstat.csv
while [ $tempo -lt 300 ]
do
    cpu=$(pidstat -u | awk '/dropbox/{print $10}')
    minflt=$(pidstat -r | awk '/dropbox/{print $5}')
    majflt=$(pidstat -r | awk '/dropbox/{print $6}')
    vsz=$(pidstat -r | awk '/dropbox/{print $7}')
    rss=$(pidstat -r | awk '/dropbox/{print $8}')
    kbr=$(pidstat -d | awk '/dropbox/{print $5}')
    kbw=$(pidstat -r | awk '/dropbox/{print $6}')
    
	echo "$cpu;$minflt;$majflt;$vsz;$rss;$kbr;$kbw;" >> log-pidstat.csv
	sleep 5
	tempo=$((tempo+5))
done


# Script usando varias ferramentas diferentes
# momento da coleta, nucleos (idle), cpu total (idle), mem used, free, shared, total,  
tempo=0;
pid=$(ps aux | grep chrome | head -n 1 | awk '{print $2}')
echo "TIME;CPU-0;CPU-1;CPU-2;CPU-3;CPU-T;USED;FREE;SHARED;TOTAL;R-AWAIT;RAREQ-SZ;W-AWAIT;WAREQ-SZ;"
while [ $tempo -lt 300 ]
do
    time=$(uptime | awk '{print $1}')
    cpu0=$(mpstat -P 0 | tail -n 1 | awk '/sda/{print $13}')
    cpu1=$(mpstat -P 1 | tail -n 1 | awk '/sda/{print $13}')
    cpu2=$(mpstat -P 2 | tail -n 1 | awk '/sda/{print $13}')
    cpu3=$(mpstat -P 3 | tail -n 1 | awk '/sda/{print $13}')
    cput=$(mpstat -P ALL | tail -n 1 | awk '/sda/{print $13}')
    used=$(free -m | awk '/Mem/{print $3}')
    free=$(free -m | awk '/Mem/{print $4}')
    shared=$(free -m | awk '/Mem/{print $5}')
    total=$(free -m | awk '/Mem/{print $2}')
    rawait=$(iostat -x | awk '/sda/{print $6}')
    rareqsz=$(iostat -x | awk '/sda/{print $7}')
    wawait=$(iostat -x | awk '/sda/{print $12}')
    wareqsz=$(iostat -x | awk '/sda/{print $13}')

    echo "$time;$cpu0;$cpu1;$cpu2;$cpu3;$cput;$used;$free;$shared;$total;$rawait;$rareqsz;$wawait;$wareqsz;"
    sleep 5
    tempo=$((tempo+5))
done
